from TOOLS.dbConnection import connect_to_database
from constants import API_KEY, SERVER_URL, ESTACIONES, TIME_BETWEEN_YEARS_REQUEST, TIME_BETWEEN_STATIONS_REQUEST
from datetime import datetime

import psycopg2
import requests
import json
import time


def get_historico_aemet():
    # Directorio que contiene los archivos CSV descarga de Puertos del Estado
    directorio = "./DATA/AEMET/JSON"
    db_connection = connect_to_database()
    cur = db_connection.cursor()
    estaciones = ESTACIONES
    # segundos a esperar entre estaciones para evitar http/429
    time_between_stations = TIME_BETWEEN_STATIONS_REQUEST
    # segundos a esperar entre años para evitar http/429
    time_between_years = TIME_BETWEEN_YEARS_REQUEST
    no200 = 0
    err429 = 0
    no_data = 0
    dberrors = 0

    querystring = {"api_key": API_KEY}
    headers = {'cache-control': "no-cache"}
    registros_totales = 0

    def record_transform(data):
        new_data = {}
        for key in [
            "fecha", "indicativo", "nombre", "provincia", "altitud", "tmed", "prec",
            "tmin", "horatmin", "tmax", "horatmax", "dir", "velmedia", "racha",
            "horaracha", "sol", "presmax", "horapresmax", "presmin", "horapresmin",
            "hrmedia", "hrmax", "horahrmax", "hrmin", "horahrmin"
        ]:
            if key in data:
                value = data[key]
                if key == "fecha":
                    try:
                        new_data[key] = datetime.strptime(value, "%Y-%m-%d").timestamp()
                    except ValueError:
                        new_data[key] = None
                elif isinstance(value, str) and ',' in value:
                    try:
                        new_data[key] = float(value.replace(',', '.'))
                    except ValueError:
                        new_data[key] = value  # Si no se puede convertir a float, asignar value
                elif value == "Ip":  # (Ip = inferior a 0,1 mm, por lo que le asigo 0.09 para la carga)
                    new_data[key] = 0.09
                elif value == "Varias":
                    new_data[key] = "Many"  # Reemplazar "Varias" por "Many" para mantener un varchar(5)
                else:
                    new_data[key] = value
            else:
                new_data[key] = None  # Si el campo no existe en el registro, asignar valor None
        return tuple(new_data.values())  # Convertir el diccionario en una tupla de valores

    for estacion in estaciones:
        registros_estacion = 0
        for year in range(2013, 2024):
            fechaini = f"{year}-01-01T00:00:00UTC"
            fechafin = f"{year}-12-31T23:59:59UTC"

            url = f"{SERVER_URL}/valores/climatologicos/diarios/datos" + \
                  f"/fechaini/{fechaini}/fechafin/{fechafin}/estacion/{estacion}"
            response = requests.request("GET", url, headers=headers, params=querystring)
            print("URL:" + url)
            # Comprobar si la solicitud fue exitosa (estado 200)
            if response.json()["estado"] == 200:
                # Obtener la URL real de los datos
                url_data = response.json()["datos"]
                # Realizar una nueva solicitud GET a la URL de los datos
                data_response = requests.get(url_data, headers=headers, params=querystring)
                if data_response.status_code == 200:
                    # Convertir la respuesta JSON a un objeto Python
                    data_json = json.loads(data_response.text)
                    with open(directorio + "/" + estacion + "_" + str(year) + ".json", "w") as archivo:
                        # Iterar sobre los datos y mostrar la información
                        for climaDiario in data_json:
                            # Insertar los datos de la estación en la tabla correspondiente
                            try:
                                print(climaDiario)
                                record_cleaned = record_transform(climaDiario)
                                print(record_cleaned)
                                cur.execute("""
                                    INSERT INTO AEMET_HISTORICO (fecha, indicativo, nombre, provincia, altitud, tmed, prec,
                                    tmin, horatmin, tmax, horatmax, dir, velmedia, racha, 
                                    horaracha, sol, presmax, horapresmax, presmin, horapresmin,
                                    hrmedia, hrmax, horahrmax, hrmin, horahrmin) 
                                    VALUES (to_timestamp(%s), %s, %s, %s, %s, %s::float, %s::float, %s::float, %s, %s::float, %s, %s::float,
                                    %s::float, %s::float, %s, %s::float, %s::float, %s, %s::float, %s, %s::float, %s::float, %s,
                                    %s::float, %s::float)
                                    """, record_cleaned)
                                archivo.write(json.dumps(climaDiario) + '\n')
                                registros_estacion += 1
                            except psycopg2.Error as e:
                                dberrors += 1
                                print(f"[{dberrors}]", "Error al insertar datos de la estación:",
                                      climaDiario["indicativo"],
                                      e)
                                exit(e.pgcode)
                    db_connection.commit()
                    print("Datos de la estación", climaDiario["indicativo"],
                          "insertados correctamente.",
                          "[", registros_estacion, "]")
                    print("Esperando", time_between_years, "segundos...")
                    time.sleep(time_between_years)
            elif response.json()["estado"] == 429 or response.status_code == 429:
                err429 += 1
                print("No hay datos para el " + f"{year}")
            elif response.json()["estado"] == 404:
                no_data += 1
                print("No hay datos para el " + f"{year}")
            elif response.status_code != 200:
                no200 += 1
                print("Error", response.status_code)
        print("Esperando", time_between_stations, "segundos...")
        time.sleep(time_between_stations)
    print("Borrando los registros con valores en campos relevantes a nulo")
    cur.execute("""delete from aemet_historico ah where
                    ah.dir is null or 
                    ah.velmedia is null or 
                    ah.racha is null""")
    db_connection.commit()
    cur.close()
    db_connection.close()

    print("Errors report:")
    print("No HTTP/200:" + f"{no200}")
    print("HTTP/429:" + f"{err429}")
    print("HTTP/404:" + f"{no_data}")
    print("DBErrors:" + f"{dberrors}")
