import os

import numpy as np
import pandas as pd
from TOOLS.dbConnection import connect_to_database


def get_ports_data():
    # Directorio que contiene los archivos CSV descarga de Puertos del Estado
    directorio = './DATA/PUERTOS/CSV'

    # Conexión a la base de datos
    conn = connect_to_database()

    # Abrir un cursor para ejecutar consultas
    cur = conn.cursor()

    # DataFrames para la carga de los ficheros por separado
    ALL = pd.DataFrame()
    SALINITY = pd.DataFrame()
    WIND = pd.DataFrame()
    CURRENTS = pd.DataFrame()
    WAVE = pd.DataFrame()
    WATER_TEMP = pd.DataFrame()

    def to_null(value):
        return None if value == -9999.9 else value

    # Generación de los df concatenados por tipo y agregando el punto del modelo de donde se obtiene
    for archivo in os.listdir(directorio):
        # Carga de ficheros
        if archivo.endswith('.csv'):
            print(archivo)
            # Extraer el número de punto del nombre del archivo y el tipo de métrica
            punto = archivo.split('_')[2]
            tipo = archivo.split('_')[3]
            # Leer el archivo CSV y agregar el punto geográfico
            DataArchivo = pd.read_csv(os.path.join(directorio, archivo), sep='\t', skiprows=1)
            DataArchivo['PuntoModelo'] = punto

            if tipo == 'ALL':
                ALL = pd.concat([ALL, DataArchivo], ignore_index=True)
            elif tipo == 'SALINITY':
                SALINITY = pd.concat([SALINITY, DataArchivo], ignore_index=True)
            elif tipo == 'WIND':
                WIND = pd.concat([WIND, DataArchivo], ignore_index=True)
            elif tipo == 'CURRENTS':
                CURRENTS = pd.concat([CURRENTS, DataArchivo], ignore_index=True)
            elif tipo == 'WAVE':
                WAVE = pd.concat([WAVE, DataArchivo], ignore_index=True)
            elif tipo == 'WATER':
                WATER_TEMP = pd.concat([WATER_TEMP, DataArchivo], ignore_index=True)
    del DataArchivo

    dataframes_names = [name for name in globals() if isinstance(globals()[name], pd.DataFrame)]
    dataframes = []
    for df_name in dataframes_names:
        # Convertir la columna 'Fecha (GMT)' a formato de fecha y hora en cada DataFrame
        globals()[df_name]['Fecha (GMT)'] = pd.to_datetime(globals()[df_name]['Fecha (GMT)'], format='%Y %m %d %H')

        # Añadir el DataFrame modificado a la lista de DataFrames
        dataframes.append(globals()[df_name])

    # Generando dataframe con todos los datos
    data_ports = pd.merge(WAVE, WIND, on=['PuntoModelo', 'Fecha (GMT)'])
    data_ports = pd.merge(data_ports, CURRENTS, on=['PuntoModelo', 'Fecha (GMT)'])
    data_ports = pd.merge(data_ports, WATER_TEMP, on=['PuntoModelo', 'Fecha (GMT)'])
    data_ports = pd.merge(data_ports, SALINITY, on=['PuntoModelo', 'Fecha (GMT)'])

    # Modelando df ALL para poder concatenar
    ALL = pd.concat([ALL, pd.DataFrame({
        'Velocidad media de Corriente(m/s)': -9999.9,
        'Dir. de prop. de la Corriente (0=N,90=E)': -9999.9,
        'Temperatura del Agua(ºC)': -9999.9,
        'Salinidad(psu)': -9999.9
    }, index=ALL.index)], axis=1)

    # Concatenando ALL al df final
    data_ports = pd.concat([data_ports, ALL], axis=0)
    data_ports['Fecha (GMT)'] = pd.to_datetime(data_ports['Fecha (GMT)'], format='%Y %m %d %H')
    # Reemplazo -9999.9 por nulos según la documentación de puertos.es
    data_ports.replace(-9999.9, np.nan, inplace=True)
    print(data_ports.isnull().sum)

    # Estudiando valores nulos por columna
    for columna in data_ports.columns:
        cantidad_nulos = data_ports[columna].isnull().sum()
        porcentaje_nulos = (cantidad_nulos / len(data_ports[columna])) * 100
        print(f"{columna}: {cantidad_nulos} valores nulos [{porcentaje_nulos:.2f}%]")

    columnas_a_verificar = [
        'Fecha (GMT)',
        'Altura Signif. del Oleaje(m)',
        'Periodo Medio(s)',
        'Periodo de Pico(s)',
        'Direcc. Media de Proced.(0=N,90=E)',
        'PuntoModelo',
        'Velocidad del viento(m/s)',
        'Direc. de proced. del Viento(0=N,90=E)'
    ]

    # Elimino los nulos de las columnas que realmente son relevantes
    data_ports = data_ports.dropna(subset=columnas_a_verificar, how='any')

    # Contador para llevar el registro de los registros insertados
    registros_insertados = 0

    # Iterar sobre cada registro y insertar en la base de datos
    for index, row in data_ports.iterrows():
        record_cleaned = tuple(to_null(row[column]) for column in data_ports.columns)
        cur.execute("""
            INSERT INTO PUERTOS_HISTORICO (fecha, altura_oleaje, periodo_medio_oleaje, 
            periodo_pico_oleaje, direccion_oleaje, altura_mar_viento, direccion_mar_viento, 
            altura_mar_fondo_1, periodo_medio_mar_fondo_1, direccion_mar_fondo_1,
            altura_mar_fondo_2, periodo_medio_mar_fondo_2, direccion_mar_fondo_2, punto_modelo, velocidad_viento,
            direccion_viento, velocidad_corriente, direccion_corriente, temperatura_agua, salinidad)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
        """, record_cleaned)
        # Incrementar el contador de registros insertados
        registros_insertados += 1
        # Mostrar por pantalla el progreso cada 5000 registros insertados
        if registros_insertados % 5000 == 0:
            conn.commit()
            print(f"Registros insertados en HISTORICO_PUERTOS hasta ahora: {registros_insertados}")

    conn.commit()

    # Cerrar cursor
    cur.close()
    conn.close()

    print(f"Registros insertados en HISTORICO_PUERTOS: {registros_insertados}")

    # data_ports.to_json("../../JSON/portsData.json", orient="records")

    # data_ports es el conjunto de datos obtenido de Puertos del Estado en un solo df listo para etiquetar
