from TOOLS.dbConnection import connect_to_database
from TOOLS.funciones import list_to_sqlstrlist
from constants import ESTACIONES
from psycopg2 import Error
from TOOLS.formulas import distance_by_haversine

from TOOLS.dbConnection import connect_to_database
from constants import ESTACIONES


def get_spot_relations():
    try:
        # Conectar a la base de datos
        db_connection = connect_to_database()

        # Consulta de spots
        spots = db_connection.cursor()
        sql_spots = "SELECT * FROM WC_SPOT"
        spots.execute(sql_spots)
        spots_cols = [desc[0] for desc in spots.description]
        spots_results = spots.fetchall()

        # Consulta de estaciones
        estaciones = db_connection.cursor()
        sql_estaciones = ("SELECT * FROM AEMET_ESTACION WHERE INDICATIVO IN (" +
                          list_to_sqlstrlist(ESTACIONES) + ");")
        estaciones.execute(sql_estaciones)
        estaciones_cols = [desc[0] for desc in estaciones.description]
        estaciones_results = estaciones.fetchall()

        # Cursor para insertar en wc_spot_aemet_estacion
        spotestacion = db_connection.cursor()

        # Consulta de puntos modelo
        puntos = db_connection.cursor()
        sql_puntos = "SELECT * FROM PUERTOS_PUNTO_MODELO;"
        puntos.execute(sql_puntos)
        puntos_cols = [desc[0] for desc in puntos.description]
        puntos_results = puntos.fetchall()

        # Cursor para insertar en wc_spot_puertos_punto_modelo
        spotpunto = db_connection.cursor()
        spotpunto.execute("""DELETE FROM wc_spot_puertos_punto_modelo""")
        db_connection.commit()

        # Recorrer los spots
        for spot in spots_results:
            # Buscar la estación más cercana al spot
            distancia_minima_estacion = float('inf')
            for estacion in estaciones_results:
                distancia = distance_by_haversine(estacion[estaciones_cols.index('latitud')],
                                                  estacion[estaciones_cols.index('longitud')],
                                                  spot[spots_cols.index('latitud')],
                                                  spot[spots_cols.index('longitud')])
                if distancia < distancia_minima_estacion:
                    distancia_minima_estacion = distancia
                    estacion_cercana = estacion[estaciones_cols.index('indicativo')]
            # Consulta para insertar registros en la tabla wc_spot_aemet_estacion
            insert_query = "INSERT INTO wc_spot_aemet_estacion (id_spot, id_estacion) VALUES (%s, %s)"

            print("Insertando", spot[spots_cols.index('id')],
                  "con estacion", estacion_cercana,
                  "con distancia", distancia_minima_estacion)
            spotestacion.execute(insert_query,
                                 (spot[spots_cols.index('id')], estacion_cercana))
            db_connection.commit()

            # Buscar el punto del modelo más cercano al spot
            distancia_minima_punto = float('inf')
            for punto in puntos_results:
                distancia = distance_by_haversine(punto[puntos_cols.index('latitud')],
                                                  punto[puntos_cols.index('longitud')],
                                                  spot[spots_cols.index('latitud')],
                                                  spot[spots_cols.index('longitud')])
                if distancia < distancia_minima_punto:
                    distancia_minima_punto = distancia
                    punto_cercano = punto[puntos_cols.index('punto_modelo')]
            # Consulta para insertar registros en la tabla wc_spot_aemet_estacion
            insert_query = "INSERT INTO wc_spot_puertos_punto_modelo (id_spot, punto_modelo) VALUES (%s, %s)"

            print("Insertando", spot[spots_cols.index('id')],
                  "con punto", punto_cercano,
                  "con distancia", distancia_minima_punto)
            spotpunto.execute(insert_query,
                              (spot[spots_cols.index('id')], punto_cercano))
            db_connection.commit()

    except (Exception, Error) as error:
        print("Error al trabajar con PostgreSQL:", error)

    finally:
        # Cerrar la conexión y el estacionessor
        if db_connection:
            db_connection.commit()
            spots.close()
            spotestacion.close()
            spotpunto.close()
            puntos.close()
            estaciones.close()
            db_connection.close()
            print("Conexión con PostgreSQL cerrada")
