from TOOLS.ml_func import *
from constants import PARAM_RANDOM_XGB, PARAM_RANDOM_RF, PARAM_GRID_XGB, PARAM_GRID_RF

os.environ['TF_CPP_MIN_LOG_LEVEL'] = '1'


def do_testmodels():
    df_valoracion, df_sail_size, df_wind_board_size = do_prepdfs(1, new=True)

    # print("\nOpción 1:")
    print_metrics(df_valoracion, 'valoracion', _return=False)
    op1_metrics_sail = print_metrics(df_sail_size, 'sail_size', 'Op1_')
    op1_metrics_board = print_metrics(df_wind_board_size, 'wind_board_size', 'Op1_')

    # Evaluación Opción 2 (Incremental), los df contendrán todas las variables más las variables objetivo que se han
    # predicho anteriormente en tiempo de ejecución del algoritmo, pudiendo así pasarlas como entrada
    # siguiendo el orden valoracion, sail_size y wind_board_size
    df_valoracion, df_sail_size, df_wind_board_size = do_prepdfs(2)

    # print("\nOpción 2:")
    op2_metrics_sail = print_metrics(df_sail_size, 'sail_size', 'Op2_')
    op2_metrics_board = print_metrics(df_wind_board_size, 'wind_board_size', 'Op2_')

    # plot_model_comparison_dual(op1_metrics_val, op2_metrics_val, 'Comparativa Opciones [valoracion]')
    plot_model_comparison_dual(op1_metrics_sail, op2_metrics_sail, '[sail_size]')
    plot_model_comparison_dual(op1_metrics_board, op2_metrics_board, '[wind_board_size]')


def do_tunning():
    df_valoracion, df_sail_size, df_wind_board_size = do_prepdfs(2)

    # Tunning modelo variable valoracion
    params_search('v1_', df_valoracion, 'valoracion', PARAM_RANDOM_RF,
                  'randomforest', 'random', FOLDS)
    params_search('v1_', df_valoracion, 'valoracion', PARAM_GRID_RF,
                  'randomforest', 'grid', FOLDS)

    # Tunning modelo variable sail_size
    params_search('v1_', df_sail_size, 'sail_size', PARAM_RANDOM_XGB,
                  'xgbregressor', 'random', FOLDS)
    params_search('v1_', df_sail_size, 'sail_size', PARAM_GRID_XGB,
                  'xgbregressor', 'grid', FOLDS)

    # Tunning modelo variable wind_board_size
    params_search('v1_', df_wind_board_size, 'wind_board_size', PARAM_RANDOM_RF,
                  'randomforest', 'random', FOLDS)
    params_search('v1_', df_wind_board_size, 'wind_board_size', PARAM_GRID_RF,
                  'randomforest', 'grid', FOLDS)


def do_visual_validation():
    # input_data --> peso_rider, velocidad_viento, edad_rider, racha, periodo_medio_oleaje
    input_data = np.array([  # Val     sail    wbs
        [68, 0, 60, 12, 6],  # 1.0     null    null
        [66, 32, 19, 37, 3],  # 1.0     null    null
        [81, 12, 57, 29, 6],  # 1.0     null    null
        [92, 17, 55, 25, 3],  # 1.0     null    null
        [75, 7, 15, 13, 3],  # 1.0     null    null
        [91, 14, 24, 26, 4],  # 1.0     null    null
        [76, 13, 38, 39, 6],  # 1.0     null    null
        [84, 10, 61, 35, 5],  # 1.0     null    null
        [86, 4, 42, 17, 6],  # 1.0     null    null
        [82, 16, 58, 24, 4],  # 1.0     null    null
        [90, 17, 26, 26, 4],  # 2.0     6.5     100.0
        [93, 20, 44, 31, 3],  # 2.0     6.5     115.0
        [79, 18, 64, 35, 4],  # 2.0     6.0     90.0
        [94, 19, 30, 46, 4],  # 2.0     6.5     105.0
        [77, 20, 30, 27, 4],  # 2.0     6.0     90.0
        [95, 16, 28, 32, 4],  # 2.0     6.5     105.0
        [74, 17, 14, 25, 3],  # 2.0     5.5     85.0
        [86, 19, 20, 25, 4],  # 2.0     6.5     100.0
        [67, 20, 43, 25, 4],  # 2.0     5.0     80.0
        [67, 18, 46, 32, 5],  # 2.0     4.0     80.0
        [94, 23, 28, 35, 5],  # 3.0     6.5     95.0
        [71, 22, 22, 24, 3],  # 3.0     4.7     80.0
        [89, 22, 33, 36, 5],  # 3.0     6.0     90.0
        [96, 24, 25, 28, 3],  # 3.0     6.5     105.0
        [89, 24, 30, 27, 4],  # 3.0     6.0     95.0
        [84, 22, 39, 29, 5],  # 3.0     5.5     85.0
        [92, 22, 24, 36, 5],  # 3.0     6.0     95.0
        [83, 24, 53, 39, 5],  # 3.0     5.5     85.0
        [90, 24, 26, 33, 5],  # 3.0     6.0     90.0
        [91, 22, 34, 38, 5],  # 3.0     6.0     95.0
        [71, 23, 64, 32, 4],  # 4.0     4.7     80.0
        [97, 23, 15, 37, 3],  # 4.0     6.5     105.0
        [77, 28, 18, 37, 6],  # 4.0     5.0     80.0
        [82, 22, 48, 31, 3],  # 4.0     5.5     90.0
        [97, 28, 50, 33, 5],  # 4.0     6.5     100.0
        [67, 17, 46, 28, 5],  # 4.0     4.0     75.0
        [96, 25, 21, 27, 5],  # 4.0     6.5     100.0
        [71, 25, 42, 46, 5],  # 4.0     4.7     80.0
        [92, 26, 46, 31, 5],  # 4.0     6.0     95.0
        [72, 25, 58, 36, 4],  # 4.0     4.7     75.0
        [68, 25, 28, 30, 5],  # 5.0     4.5     70.0
        [97, 25, 55, 34, 5],  # 5.0     6.5     100.0
        [96, 25, 42, 41, 6],  # 5.0     6.5     100.0
        [74, 28, 31, 38, 5],  # 5.0     5.0     75.0
        [88, 24, 63, 48, 5],  # 5.0     6.0     90.0
        [84, 28, 62, 48, 5],  # 5.0     5.5     85.0
        [96, 25, 53, 48, 5],  # 5.0     6.5     100.0
        [78, 25, 28, 37, 6],  # 5.0     5.0     80.0
        [89, 28, 40, 36, 5],  # 5.0     6.0     90.0
        [89, 28, 55, 44, 5]  # 5.0     6.0     90.0
    ])

    for _input in input_data:
        peso, vel_viento, edad, racha, periodo = _input
        # Llamar a la función windcaddyalg con los valores actuales
        resultado = windcaddyalg(peso, vel_viento, edad, racha, periodo)
        print("Datos de entrada:", _input)
        print("Resultado:", resultado)
