--TRANSFORMA GRADOS SEXSAGESIMALES A DIRECCIONES DE LA ROSA DE VIENTOS
CREATE OR REPLACE FUNCTION grad2Dir(grados DOUBLE PRECISION)
RETURNS VARCHAR AS $$
BEGIN
    IF grados >= 0 AND grados < 11.25 THEN
        RETURN 'N';
    ELSIF grados >= 11.25 AND grados < 33.75 THEN
        RETURN 'NNE';
    ELSIF grados >= 33.75 AND grados < 56.25 THEN
        RETURN 'NE';
    ELSIF grados >= 56.25 AND grados < 78.75 THEN
        RETURN 'ENE';
    ELSIF grados >= 78.75 AND grados < 101.25 THEN
        RETURN 'E';
    ELSIF grados >= 101.25 AND grados < 123.75 THEN
        RETURN 'ESE';
    ELSIF grados >= 123.75 AND grados < 146.25 THEN
        RETURN 'SE';
    ELSIF grados >= 146.25 AND grados < 168.75 THEN
        RETURN 'SSE';
    ELSIF grados >= 168.75 AND grados < 191.25 THEN
        RETURN 'S';
    ELSIF grados >= 191.25 AND grados < 213.75 THEN
        RETURN 'SSO';
    ELSIF grados >= 213.75 AND grados < 236.25 THEN
        RETURN 'SO';
    ELSIF grados >= 236.25 AND grados < 258.75 THEN
        RETURN 'OSO';
    ELSIF grados >= 258.75 AND grados < 281.25 THEN
        RETURN 'O';
    ELSIF grados >= 281.25 AND grados < 303.75 THEN
        RETURN 'ONO';
    ELSIF grados >= 303.75 AND grados < 326.25 THEN
        RETURN 'NO';
    ELSIF grados >= 326.25 AND grados < 348.75 THEN
        RETURN 'NNO';
    ELSE
        RETURN 'N'; -- Se considera que 360° es igual a 0°
    END IF;
END;
$$ LANGUAGE plpgsql;

-- CREACIÓN DEL DATASET
drop table if exists wc_labeldataset ;

CREATE TABLE wc_labeldataset (
    id SERIAL PRIMARY KEY,
    id_spot INT,
    spot VARCHAR(255),
    fecha DATE,
    dir VARCHAR(255),
    velmedia FLOAT,
    racha FLOAT,
    altura_oleaje FLOAT,
    periodo_medio_oleaje FLOAT,
    periodo_pico_oleaje FLOAT,
    direccion_oleaje FLOAT,
    velocidad_viento FLOAT,
    direccion_viento FLOAT,
    velocidad_corriente FLOAT,
    direccion_corriente FLOAT,
    id_deporte INT,
    deporte VARCHAR(255),
    id_modalidad INT,
    modalidad VARCHAR(255),
    valoracion FLOAT,
    perfil_rider VARCHAR(255),
    peso_rider FLOAT,
    edad_rider FLOAT,
    kite_size FLOAT,
    sail_size FLOAT,
    wing_size FLOAT,
    kite_board_size FLOAT,
    kite_board_type VARCHAR(255),
    wind_board_size FLOAT,
    wind_board_type VARCHAR(255),
    wing_board_size FLOAT,
    wing_ffoil_size FLOAT,
    labeled BOOLEAN default false
);

create index wc_labeldataset_labeled_index
    on wc_labeldataset (labeled);

--SOLO INSERTO DATOS DE MEDICIONES DESDE LAS 07:00 A 20:00
--NADIE VA AL AGUA ANTES DE LAS 07:00 NI DESPUES DE LAS 20:00
INSERT INTO wc_labeldataset (id, id_spot, spot, fecha, dir, velmedia, racha, altura_oleaje, periodo_medio_oleaje,
periodo_pico_oleaje, direccion_oleaje, velocidad_viento, direccion_viento, velocidad_corriente, direccion_corriente,
id_deporte, deporte, id_modalidad, modalidad, valoracion, perfil_rider, peso_rider, edad_rider, kite_size, sail_size, wing_size, kite_board_size,
kite_board_type, wind_board_size, wind_board_type, wing_board_size, wing_ffoil_size, labeled)
SELECT
  ROW_NUMBER() OVER () AS id,
  ws.id as id_spot,
  ws.nombre as spot,
  ph.fecha,
  ah.dir,
  round(ah.velmedia * 1.94384) as velmedia,
  round(ah.racha * 1.94384) as racha,
  ph.altura_oleaje,
  round(ph.periodo_medio_oleaje) as periodo_medio_oleaje,
  round(ph.periodo_pico_oleaje) as periodo_pico_oleaje,
  ph.direccion_oleaje,
  round(ph.velocidad_viento * 1.94384) as velocidad_viento,
  ph.direccion_viento,
  round(ph.velocidad_corriente) as velocidad_corriente,
  ph.direccion_corriente,
  wd.id as id_deporte,
  wd.deporte as deporte,
  wm.id as id_modalidad,
  wm.modalidad as modalidad,
  null as valoracion,
  null as perfil_rider,
  null as peso_rider,
  null as edad_rider,
  null as kite_size,
  null as sail_size,
  null as wing_size,
  null as kite_board_size,
  null as kite_board_type,
  null as wind_board_size,
  null as wind_board_type,
  null as wing_board_size,
  null as wing_ffoil_size,
  false as labeled
FROM
  wc_spot ws,
  wc_spot_puertos_punto_modelo wsppm,
  wc_spot_aemet_estacion wsae,
  aemet_historico ah,
  puertos_historico ph,
  wc_deporte wd,
  wc_modalidad wm
WHERE
  ws.id = wsppm.id_spot
  AND ws.id = wsae.id_spot
  AND date_trunc('day', ah.fecha) = date_trunc('day', ph.fecha)
  AND ah.indicativo = wsae.id_estacion
  AND ph.punto_modelo = wsppm.punto_modelo
  and ph.velocidad_viento < ah.racha --> La velocidad del viento no puede ser mayor a la racha
  AND EXTRACT(HOUR FROM ph.fecha::timestamp) between 7 and 20
;

comment on column wc_labeldataset.labeled is 'indica si el registro se ha evaluado';
COMMENT ON COLUMN wc_labeldataset.id IS 'Identificador registro';
COMMENT ON COLUMN wc_labeldataset.id_spot IS 'Identificador spot';
COMMENT ON COLUMN wc_labeldataset.spot IS 'Nombre spot';
COMMENT ON COLUMN wc_labeldataset.fecha IS 'Fecha del día (AAAA-MM-DD), convertida a TIMESTAMP';
COMMENT ON COLUMN wc_labeldataset.dir IS 'Dirección de la racha máxima en decenas de grado (99 = dirección variable) (88 = sin dato)';
COMMENT ON COLUMN wc_labeldataset.velmedia IS 'Velocidad media del viento en nudos';
COMMENT ON COLUMN wc_labeldataset.racha IS 'Racha máxima del viento en nudos';
COMMENT ON COLUMN wc_labeldataset.altura_oleaje IS 'Altura Signif. del Oleaje(m)';
COMMENT ON COLUMN wc_labeldataset.periodo_medio_oleaje IS 'Periodo Medio(s)';
COMMENT ON COLUMN wc_labeldataset.periodo_pico_oleaje IS 'Periodo Pico(s)';
COMMENT ON COLUMN wc_labeldataset.direccion_oleaje IS 'Direcc. Media de Proced.(0=N,90=E)';
COMMENT ON COLUMN wc_labeldataset.velocidad_viento IS 'Velocidad del viento(knots)';
COMMENT ON COLUMN wc_labeldataset.direccion_viento IS 'Direc. de proced. del Viento(0=N,90=E)';
COMMENT ON COLUMN wc_labeldataset.velocidad_corriente IS 'Velocidad corriente(m/s)';
COMMENT ON COLUMN wc_labeldataset.direccion_corriente IS 'Dir. de prop. de la Corriente (0=N,90=E)';

-- TABLA PARA BACKUP DE DATOS POR SI HAY ERROR HUMANO
CREATE TABLE IF NOT EXISTS wc_labeldataset_bck as
    select * from wc_labeldataset;

ALTER TABLE wc_labeldataset_bck ADD PRIMARY KEY (ID);

create index wc_labeldataset_bck_labeled_index
    on wc_labeldataset_bck (labeled);

comment on column wc_labeldataset_bck.labeled is 'indica si el registro se ha evaluado';
COMMENT ON COLUMN wc_labeldataset_bck.id IS 'Identificador registro';
COMMENT ON COLUMN wc_labeldataset_bck.id_spot IS 'Identificador spot';
COMMENT ON COLUMN wc_labeldataset_bck.spot IS 'Nombre spot';
COMMENT ON COLUMN wc_labeldataset_bck.fecha IS 'Fecha del día (AAAA-MM-DD), convertida a TIMESTAMP';
COMMENT ON COLUMN wc_labeldataset_bck.dir IS 'Dirección de la racha máxima en decenas de grado (99 = dirección variable) (88 = sin dato)';
COMMENT ON COLUMN wc_labeldataset_bck.velmedia IS 'Velocidad media del viento en nudos';
COMMENT ON COLUMN wc_labeldataset_bck.racha IS 'Racha máxima del viento en nudos';
COMMENT ON COLUMN wc_labeldataset_bck.altura_oleaje IS 'Altura Signif. del Oleaje(m)';
COMMENT ON COLUMN wc_labeldataset_bck.periodo_medio_oleaje IS 'Periodo Medio(s)';
COMMENT ON COLUMN wc_labeldataset_bck.periodo_pico_oleaje IS 'Periodo Pico(s)';
COMMENT ON COLUMN wc_labeldataset_bck.direccion_oleaje IS 'Direcc. Media de Proced.(0=N,90=E)';
COMMENT ON COLUMN wc_labeldataset_bck.velocidad_viento IS 'Velocidad del viento(knots)';
COMMENT ON COLUMN wc_labeldataset_bck.direccion_viento IS 'Direc. de proced. del Viento(0=N,90=E)';
COMMENT ON COLUMN wc_labeldataset_bck.velocidad_corriente IS 'Velocidad corriente(m/s)';
COMMENT ON COLUMN wc_labeldataset_bck.direccion_corriente IS 'Dir. de prop. de la Corriente (0=N,90=E)';

-- ESTE PROCEDURE RECOJE 10k MUESTRA DE CONDICIONES PARA ETIQUETAR
-- DIARIAMENTE AUMENTANDO EL RENDIMIENTO DE LAS CONSULTAS Y ACTUALIZACIONES
-- EN EL FORMULARIO DE ETIQUETADO.
-- RESTRINJO LOS IDS A ETIQUETAR A WINDSURF, OLAS
CREATE OR REPLACE PROCEDURE public.filltolabel()
	LANGUAGE 'plpgsql'
AS $procedure$
	BEGIN
    -- Borra la tabla si ya existe
    DROP TABLE IF EXISTS public.toLabel;

    -- Crea la tabla
    CREATE TABLE public.toLabel as SELECT id, labeled FROM public.WC_LABELDATASET
                                             where labeled = false
                                             and id_deporte = 1
                                             and id_modalidad = 1
                                             order by random() LIMIT 10000;

   END;
$procedure$
;

CALL public.filltolabel();

-- DROP FUNCTION public.dellabeled();

-- BORRA DE LA TABLA TOLABEL, LOS REGISTROS QUE SE HAYAN ETIQUETADO
-- E INSERTA UNA COPIA DEL REGISTRO ETIQUETADO EN LA TABLA DE BACKUP
CREATE OR REPLACE FUNCTION public.dellabeled()
 RETURNS trigger
 LANGUAGE plpgsql
AS $function$
BEGIN

    -- Borra de los registros a etiquetar
	DELETE FROM public.toLabel
    WHERE id = NEW.id;

    -- Creo copia de seguridad en wc_labeldataset_bck
    IF EXISTS (SELECT 1 FROM wc_labeldataset_bck WHERE id = NEW.id) THEN
        -- Si existe, actualizar todo el registro con los nuevos valores
        UPDATE wc_labeldataset_bck
        SET
			id_spot = NEW.id_spot,
			spot = NEW.spot,
			fecha = NEW.fecha,
			dir = NEW.dir,
			velmedia = NEW.velmedia,
			racha = NEW.racha,
			altura_oleaje = NEW.altura_oleaje,
			periodo_medio_oleaje = NEW.periodo_medio_oleaje,
			periodo_pico_oleaje = NEW.periodo_pico_oleaje,
			direccion_oleaje = NEW.direccion_oleaje,
			velocidad_viento = NEW.velocidad_viento,
			direccion_viento = NEW.direccion_viento,
			velocidad_corriente = NEW.velocidad_corriente,
			direccion_corriente = NEW.direccion_corriente,
			id_deporte = NEW.id_deporte,
			deporte = NEW.deporte,
			id_modalidad = NEW.id_modalidad,
			modalidad = NEW.modalidad,
			valoracion = NEW.valoracion,
			perfil_rider = NEW.perfil_rider,
			peso_rider = NEW.peso_rider,
			kite_size = NEW.kite_size,
			sail_size = NEW.sail_size,
			wing_size = NEW.wing_size,
			kite_board_size = NEW.kite_board_size,
			kite_board_type = NEW.kite_board_type,
			wind_board_size = NEW.wind_board_size,
			wind_board_type = NEW.wind_board_type,
			wing_board_size = NEW.wing_board_size,
			wing_ffoil_size = NEW.wing_ffoil_size,
			labeled = NEW.labeled,
			edad_rider = NEW.edad_rider
        WHERE id = NEW.id;
    ELSE
        -- Si no existe, insertar un nuevo registro con todos los valores nuevos
        INSERT INTO wc_labeldataset_bck
        SELECT NEW.*;
    END IF;

    RETURN NEW;
END;
$function$
;

-- EJECUTA dellabeled CUANDO SE ETIQUETE EL REGISTRO EN CUESTIÓN
create or REPLACE TRIGGER trigger_delLabeled
AFTER UPDATE OF labeled ON public.WC_LABELDATASET
FOR EACH ROW
EXECUTE FUNCTION public.delLabeled();

-- CREA LA FUNCION GETPESOEDAD PARA OBTENER UN HOLDOUT SET EN EL PROCESO DE VALIDACION
-- OBTIENE VALORES DE ACUERDO A UNA GAUSSIANA CENTRADA EN LA MEDIA (mu) QUE SE PARSE COMO PARAMETRO
-- Y CON UNA SD (sigma), DEVUELVE SOLO VALORES ENTRE minimo Y maximo.
CREATE OR REPLACE FUNCTION public.getpesoedad(mu double precision, sigma double precision, minimo double precision, maximo double precision)
 RETURNS text
 LANGUAGE plpgsql
AS $function$
DECLARE
    u1 FLOAT8;
    u2 FLOAT8;
    z0 FLOAT8;
    edad FLOAT8;
    peso FLOAT8;
BEGIN
    LOOP
        u1 := RANDOM();
        u2 := RANDOM();
        z0 := sqrt(-2 * ln(u1)) * cos(2 * pi() * u2);
        edad := z0 * sigma + mu;
        edad := FLOOR(edad);
        peso := 0;
        EXIT WHEN edad >= minimo AND edad <= maximo;
    END LOOP;

    CASE
        WHEN edad >= 11 AND edad < 20 THEN
            CASE
                WHEN edad = 11 THEN peso := ((35.6 - 35.2) * RANDOM()) + 35.2;
                WHEN edad = 12 THEN peso := ((39.9 - 39.5) * RANDOM()) + 39.5;
                WHEN edad = 13 THEN peso := ((44.6 - 44.0) * RANDOM()) + 44.0;
                WHEN edad = 14 THEN peso := ((49.8 - 48.5) * RANDOM()) + 48.5;
                WHEN edad = 15 THEN peso := ((55.3 - 53.1) * RANDOM()) + 53.1;
                WHEN edad = 16 THEN peso := ((60.8 - 57.4) * RANDOM()) + 57.4;
                WHEN edad = 17 THEN peso := ((64.8 - 60.6) * RANDOM()) + 60.6;
                WHEN edad >= 18 AND edad < 20 THEN peso := ((68.0 - 63.1) * RANDOM()) + 63.1;
            END CASE;
        WHEN edad >= 20 AND edad <= 29 THEN peso := ((78.0 - 68.0) * RANDOM()) + 68.0;
        WHEN edad >= 30 AND edad <= 39 THEN peso := ((80.0 - 70.0) * RANDOM()) + 70.0;
        WHEN edad >= 40 AND edad <= 49 THEN peso := ((82.0 - 72.0) * RANDOM()) + 72.0;
        WHEN edad >= 50 AND edad <= 59 THEN peso := ((84.0 - 74.0) * RANDOM()) + 74.0;
        WHEN edad >= 60 AND edad <= 69 THEN peso := ((83.0 - 74.0) * RANDOM()) + 74.0;
        WHEN edad >= 70 THEN peso := ((81.0 - 72.0) * RANDOM()) + 72.0;
    END CASE;
    peso := FLOOR(peso);
    RETURN CONCAT(edad, '|', peso);
END;
$function$
;
