import pickle

import pandas as pd
from sklearn.ensemble import RandomForestRegressor
from xgboost import XGBRegressor

from TOOLS.ml_func import do_prepdfs
from TOOLS.test_func import (evaluate_predictions, split_dataset, evaluate_models_cat,
                             apply_windcaddyalg)
from constants import RANDOM_STATE, MODELS_DIR, DEVELOP_V2_V3, VERSUS


def do_validation():
    # Desarrollo version V3 cambiando el enfoque de predicción para valoracion que pasa
    # de ser un problema de regresión a clasificación
    if DEVELOP_V2_V3:
        do_windcaddy_v2_v3()

    if VERSUS:
        # VALE HAY QUE ASEGURARSE QUE V1 ESTÉ EN LAS MISMAS CONDICIONES QUE V3
        # EN EL ENTRENAMIENTO DE V1 HAY QUE HACER QUE USE LOS DF DE WC_TRAIN_TEST_DATA
        test_df = pd.read_csv("DATA/WC_TRAIN_TEST_DATA/test_df.csv")
        test_df['pred_valoracion'] = None
        test_df['pred_sail_size'] = None
        test_df['pred_wind_board_size'] = None

        print("\nAplicando windcaddy v1 a test data")
        test_df_v1 = test_df.apply(lambda row: apply_windcaddyalg('v1_', row), axis=1)
        print("\n###### windcaddy v1 Results ######")
        do_validation_version(test_df_v1, True)
        print("\nAplicando windcaddy v2 a test data")
        test_df_v2 = test_df.apply(lambda row: apply_windcaddyalg('v2_', row), axis=1)
        print("\n###### windcaddy v2 Results ######")
        do_validation_version(test_df_v2)
        print("\nAplicando windcaddy v3 a test data")
        test_df_v3 = test_df.apply(lambda row: apply_windcaddyalg('v3_', row), axis=1)
        print("\n###### windcaddy v3 Results ######")
        do_validation_version(test_df_v3)


def do_validation_version(df, v1=False):
    print("Doing validation")

    print("Valoracion")
    if v1:
        evaluate_predictions(df["pred_valoracion"], df["valoracion"], "cont")
        evaluate_predictions(df["pred_valoracion"], df["valoracion"], "cat")
    else:
        evaluate_predictions(df["pred_valoracion"], df["valoracion"], "cat")

    # La vela y tabla solo se calculan cuando la valoracion es != 1 ("Ni me tiro"), por lo que
    # filtro el df a evaluar para que calcule métricas de los registros donde se ha predicho
    # Una vela y tabla tanto en truth como en predicted

    df_cont = df[
        (df['valoracion'] != 1) &
        (df['pred_valoracion'] != 1)].dropna()

    print("sail_size")
    evaluate_predictions(df_cont["pred_sail_size"], df_cont["sail_size"], "cont")
    print("")
    print("wind_board_size")
    evaluate_predictions(df_cont["pred_wind_board_size"], df_cont["wind_board_size"], "cont")


def do_windcaddy_v2_v3():
    train_df_val, train_df_sail, train_df_wbs = do_prepdfs(1)

    X_train, X_test, y_train, y_test = split_dataset(train_df_val, "valoracion")
    evaluate_models_cat(X_train, X_test, y_train, y_test)

    # BUILDING V2
    # ++++ Option 1: ++++ #
    ### SAIL_SIZE ###
    X_train, X_test, y_train, y_test = split_dataset(train_df_sail, "sail_size")
    sail_model = XGBRegressor(random_state=RANDOM_STATE)
    sail_model.fit(X_train, y_train)
    with open(MODELS_DIR + "v2_op1_XGBRegressor_sail_size" + ".pkl", "wb") as file:
        pickle.dump(sail_model, file)

    ### WIND_BOARD_SIZE ###
    X_train, X_test, y_train, y_test = split_dataset(train_df_wbs, "wind_board_size")
    wbs_model = RandomForestRegressor(random_state=RANDOM_STATE)
    wbs_model.fit(X_train, y_train)
    with open(MODELS_DIR + "v2_op1_Random Forest_wind_board_size" + ".pkl", "wb") as file:
        pickle.dump(wbs_model, file)

    # BUILDING V3
    # ++++ Option 2: ++++ #
    train_df_val, df_cont_sail, df_cont_wbs = do_prepdfs(2)

    ### SAIL_SIZE ###
    X_train, X_test, y_train, y_test = split_dataset(df_cont_sail, "sail_size")
    sail_model = XGBRegressor(random_state=RANDOM_STATE)
    sail_model.fit(X_train, y_train)
    with open(MODELS_DIR + "v3_op2_XGBRegressor_sail_size" + ".pkl", "wb") as file:
        pickle.dump(sail_model, file)

    ### WIND_BOARD_SIZE ###
    X_train, X_test, y_train, y_test = split_dataset(df_cont_wbs, "wind_board_size")
    wbs_model = RandomForestRegressor(random_state=RANDOM_STATE)
    wbs_model.fit(X_train, y_train)
    with open(MODELS_DIR + "v3_op2_Random Forest_wind_board_size" + ".pkl", "wb") as file:
        pickle.dump(wbs_model, file)
