import seaborn as sns
import matplotlib.pyplot as plt
import pandas as pd


def list_to_sqlstrlist(itemlist):
    # Convertir la lista de ESTACIONES en una cadena separada por comas
    strlst = ''
    for item in itemlist:
        strlst += "'" + item + "',"
    strlst = strlst[:-1]
    return strlst


def plot_corr_matrix(corr_matrix):
    # Gráfico
    # Configurar el tamaño del plot
    plt.figure(figsize=(14, 12))
    # Graficar la matriz de correlación usando un mapa de calor de Seaborn
    heatmap = sns.heatmap(corr_matrix, annot=True, cmap='coolwarm', fmt=".2f")
    # Rotar los nombres de las columnas en el eje x
    heatmap.set_xticklabels(heatmap.get_xticklabels(), rotation=45, horizontalalignment='right')
    # Añadir título
    plt.title('Matriz de correlación')
    # Mostrar el gráfico
    plt.show()


def tratar_nulos(dataframe):
    # Calcular el porcentaje de nulos para cada columna
    total_filas = len(dataframe)
    porcentaje_nulos = (dataframe.isnull().sum() / total_filas) * 100

    for columna, porcentaje in porcentaje_nulos.items():
        print(columna, "{:.2f}".format(porcentaje), '%')
        if porcentaje > 95:
            print("Elimino", columna)
            # Descartar la característica
            del dataframe[columna]

    return dataframe


def exploracion_univariada(dataframe):
    # Seleccionar solo las columnas numéricas
    columnas_numericas = dataframe.select_dtypes(include=['int', 'float']).columns

    # Calcular el número de filas necesarias
    # para acomodar todos los gráficos en una matriz de 3 columnas
    num_columnas = len(columnas_numericas)
    num_filas = (num_columnas + 2) // 3  # Redondeo hacia arriba

    # Configuración de estilo de Seaborn
    sns.set(style="whitegrid")

    # Crear una figura y una matriz de subgráficos
    fig, axes = plt.subplots(num_filas, 3, figsize=(15, 5 * num_filas))
    fig.subplots_adjust(hspace=0.5)  # Espacio vertical entre los subgráficos

    # Iterar sobre las columnas numéricas y generar histogramas en cada subgráfico
    for i, columna in enumerate(columnas_numericas):
        fila = i // 3
        columna_grafico = i % 3

        # Gráfico de distribución con Seaborn
        sns.histplot(dataframe[columna], ax=axes[fila, columna_grafico], kde=True)
        axes[fila, columna_grafico].set_title(f'Distribución de {columna}', fontsize=14)
        axes[fila, columna_grafico].set_xlabel('Valor', fontsize=12)
        axes[fila, columna_grafico].set_ylabel('Frecuencia', fontsize=12)
        axes[fila, columna_grafico].tick_params(axis='both', which='major', labelsize=10)

    # Ocultar los subgráficos vacíos si el número de columnas no es divisible por 3
    for i in range(num_columnas, num_filas * 3):
        fila = i // 3
        columna_grafico = i % 3
        axes[fila, columna_grafico].axis('off')

    plt.show()
