# Constants file
# SETUP APP
from scipy.stats import randint, uniform
from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
from sklearn.linear_model import LinearRegression, ElasticNet, Ridge, Lasso, LogisticRegression
from sklearn.svm import SVR
import xgboost as xgb
from sklearn.tree import DecisionTreeClassifier

DO_ETL = False
DO_EXPLORATORY_ANALYSIS = False
DO_PCA = False
DO_TESTMODELS = False
DO_TUNNING = False
DO_VVALIDATION = False
DO_VALIDATION = True
DEVELOP_V2_V3 = False
VERSUS = True

# ML CONFIG
# RIDGE MODEL CONFIG
RALPHA = 1.0
# LASSO MODEL CONFIG
LALPHA = 1.0
# ELASTICNET MODEL CONFIG
EALPHA = 0.1
EL1RATIO = 0.5
# SVR MODEL CONFIG
SVRC = 0.01
SVRPDEGREE = 2
GAMMA = 'scale'
# NN MODEL CONFIG
OPTIMIZER = 'adam'
LOSS = 'mean_squared_error'
EPOCHS = 50
BATCH_SIZE = 32
VERBOSE = 0
# ML CROSS VALIDATION CONFIG
SHUFFLE = True
FOLDS = 5
RANDOM_STATE = 42
# MODELS DIR
MODELS_DIR = 'ML/MODELS/'
# XGBOOST MODEL CONFIG

# Espacio de búsqueda Random Forest (Grid)
PARAM_GRID_RF = {
    'n_estimators': [50, 100, 150],
    'max_depth': [None, 10, 20],
    'min_samples_split': [2, 5, 10],
    'min_samples_leaf': [1, 2, 4],
    'max_features': ['auto', 'sqrt']
}

# Espacio de búsqueda XGBRegressor (Grid)
PARAM_GRID_XGB = {
    'n_estimators': [100, 200, 300],
    'max_depth': [3, 5, 7],
    'learning_rate': [0.01, 0.1, 0.3],
    'min_child_weight': [1, 3, 5],
    'subsample': [0.6, 0.8, 1.0],
    'colsample_bytree': [0.6, 0.8, 1.0],
    'gamma': [0, 0.1, 0.2],
    'reg_alpha': [0, 0.1, 0.5],
    'reg_lambda': [0, 0.1, 0.5]
}
# Espacio de búsqueda Random Forest (Random)
PARAM_RANDOM_RF = {
    'n_estimators': randint(50, 200),
    'max_depth': randint(2, 20),
    'min_samples_split': randint(2, 20),
    'min_samples_leaf': randint(1, 10),
    'max_features': ['auto', 'sqrt']
}

# Espacio de búsqueda XGBRegressor (Random)
PARAM_RANDOM_XGB = {
    'n_estimators': randint(50, 200),
    'max_depth': randint(2, 20),
    'learning_rate': uniform(0.01, 0.3),
    'min_child_weight': randint(1, 10),
    'subsample': uniform(0.6, 0.4),
    'colsample_bytree': uniform(0.6, 0.4),
    'gamma': uniform(0, 0.2),
    'reg_alpha': uniform(0, 0.5),
    'reg_lambda': uniform(0, 0.5)
}

MODELS_CONT = {
    'LinearRegression': LinearRegression(),
    'ElasticNet': ElasticNet(alpha=EALPHA, l1_ratio=EL1RATIO, random_state=RANDOM_STATE),
    'Ridge': Ridge(alpha=RALPHA),
    'Lasso': Lasso(alpha=LALPHA),
    'SVR Kernel: Linear': SVR(kernel='linear', C=SVRC),
    'SVR Kernel: Polinomic': SVR(kernel='poly', degree=SVRPDEGREE, gamma=GAMMA),
    'SVR Kernel: RBF': SVR(kernel='rbf', gamma=GAMMA),
    'Random Forest': RandomForestRegressor(random_state=RANDOM_STATE),
    'XGBRegressor': xgb.XGBRegressor(random_state=RANDOM_STATE)
}

MODELS_CAT = {
    'Logical Regression': LogisticRegression(),
    'Decision Tree Classifier': DecisionTreeClassifier(),
    'Random Forest': RandomForestClassifier(random_state=RANDOM_STATE),
    'XGB Classifier': xgb.XGBClassifier(random_state=RANDOM_STATE)
}

# DATASOURCES CONFIG
# AEMET
API_KEY = "eyJhbGciOiJIUzI1NiJ9.eyJzdWIiOiJic2FubGFvQHVvYy5lZHUiLCJqdGkiOiI4N2FhZmU5Yy1hNzBkLTQ3M2YtYjcyNy04Zjk5YzQ1MDExNTkiLCJpc3MiOiJBRU1FVCIsImlhdCI6MTcwOTM2NTgzOSwidXNlcklkIjoiODdhYWZlOWMtYTcwZC00NzNmLWI3MjctOGY5OWM0NTAxMTU5Iiwicm9sZSI6IiJ9.ukKdZUISZK-SmkKhDNuovQCH_mO3YwfuGDdJTlNjiBg"
ESTACIONES = ["C619I", "C629X", "C649I", "C659H", "C659M", "C689E"]
SERVER_URL = "https://opendata.aemet.es/opendata/api"
TIME_BETWEEN_STATIONS_REQUEST = 10
TIME_BETWEEN_YEARS_REQUEST = 4

# DOCKER LOCAL
# DB_NAME = "windcaddy"
# DB_USER = "windcaddy"
# DB_PASSWORD = "windcaddy"
# DB_HOST = "localhost"
# DB_PORT = "5432"


# BBDD PERSISTENTE (FORMULARIO WEB)
DB_NAME = "windcaddy"
DB_USER = "windcaddy"
DB_PASSWORD = "windcaddy"
DB_HOST = "192.168.1.170"
DB_PORT = "5432"
