import subprocess
import time
from TOOLS.dbConnection import exec_sqlfile, connect_to_database
from ETL.getEstaciones import get_estaciones
from ETL.getHistoricoDiario import get_historico_aemet
from ETL.getPortsData import get_ports_data
from ETL.getSpotRelations import get_spot_relations
from EXPLORATORY_ANALYSIS.doExporatoryAnalysis import do_exporatory_analysis
from PCA.doPCA import do_pca
from ML.doML import do_testmodels, do_tunning, do_visual_validation
from TEST.doTest import do_validation
from constants import DO_ETL, DO_PCA, DO_EXPLORATORY_ANALYSIS, DO_TESTMODELS, DO_TUNNING, DO_VVALIDATION, DO_VALIDATION

if DO_ETL:
    subprocess.call(["docker-compose", "-f", "docker-compose.yml", "up", "-d"])
    time.sleep(3)
    print("docker up!")

    print("getting data...")
    get_estaciones()
    get_spot_relations()
    get_historico_aemet()
    get_ports_data()
    print("ETL proccess done!")

    print("setup dataset for labeling...")
    conn = connect_to_database()
    cur = conn.cursor()
    exec_sqlfile("./SQL/create_wc_labeldataset.sql", cur)
    conn.commit()
    cur.close()
    conn.close()
    print("setup dataset done!")

if DO_EXPLORATORY_ANALYSIS:
    df = do_exporatory_analysis()

if DO_PCA:
    df = do_pca()

if DO_TESTMODELS:
    do_testmodels()

if DO_TUNNING:
    df = do_tunning()

if DO_VVALIDATION:
    do_visual_validation()

if DO_VALIDATION:
    do_validation()
