package tfg.windcaddy.Backend.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import tfg.windcaddy.Backend.domain.ToLabelEntity;
import tfg.windcaddy.Backend.domain.WcLabeldatasetEntity;
import tfg.windcaddy.Backend.services.ToLabelService;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/toLabel")
public class ToLabelController {

    private final ToLabelService toLabelService;

    @Autowired
    public ToLabelController(ToLabelService toLabelService) {
        this.toLabelService = toLabelService;
    }

    @PostMapping("/saveOrUpdate")
    public ResponseEntity<ToLabelEntity> saveOrUpdate(@RequestBody ToLabelEntity entity) {
        ToLabelEntity savedEntity = toLabelService.saveOrUpdate(entity);
        return new ResponseEntity<>(savedEntity, HttpStatus.OK);
    }

    @GetMapping("/{id}")
    public ResponseEntity<ToLabelEntity> findById(@PathVariable int id) {
        Optional<ToLabelEntity> entityOptional = toLabelService.findById(id);
        return entityOptional.map(entity -> new ResponseEntity<>(entity, HttpStatus.OK))
                .orElseGet(() -> new ResponseEntity<>(HttpStatus.NOT_FOUND));
    }

    @GetMapping("/findAll")
    public ResponseEntity<List<ToLabelEntity>> findAll() {
        List<ToLabelEntity> entities = toLabelService.findAll();
        return new ResponseEntity<>(entities, HttpStatus.OK);
    }

    @GetMapping("/delete/{id}")
    public ResponseEntity<Void> deleteById(@PathVariable int id) {
        toLabelService.deleteById(id);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @GetMapping("/getConditionToLabel")
    public ResponseEntity<WcLabeldatasetEntity> getRandomEntity() {
        WcLabeldatasetEntity randomEntity = toLabelService.getRandomEntity();
        if (randomEntity != null) {
            return new ResponseEntity<>(randomEntity, HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }
}
