package tfg.windcaddy.Backend.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import tfg.windcaddy.Backend.domain.WcLabeldatasetEntity;
import tfg.windcaddy.Backend.dto.WcLabelDatasetIdResponse;
import tfg.windcaddy.Backend.services.WcLabeldatasetService;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/wclabeldatasets")
public class WcLabeldatasetController {

    private final WcLabeldatasetService service;

    @Autowired
    public WcLabeldatasetController(WcLabeldatasetService service) {
        this.service = service;
    }

    @GetMapping
    public ResponseEntity<List<WcLabeldatasetEntity>> getAllEntities() {
        List<WcLabeldatasetEntity> entities = service.getAllEntities();
        return new ResponseEntity<>(entities, HttpStatus.OK);
    }

    @GetMapping("/{id}")
    public ResponseEntity<WcLabeldatasetEntity> getEntityById(@PathVariable("id") int id) {
        Optional<WcLabeldatasetEntity> entity = service.getEntityById(id);
        return entity.map(value -> new ResponseEntity<>(value, HttpStatus.OK))
                .orElseGet(() -> new ResponseEntity<>(HttpStatus.NOT_FOUND));
    }

    @PostMapping
    public ResponseEntity<WcLabeldatasetEntity> createEntity(@RequestBody WcLabeldatasetEntity entity) {
        WcLabeldatasetEntity createdEntity = service.createEntity(entity);
        return new ResponseEntity<>(createdEntity, HttpStatus.CREATED);
    }

    @PutMapping("/{id}")
    public ResponseEntity<WcLabeldatasetEntity> updateEntity(@PathVariable("id") int id, @RequestBody WcLabeldatasetEntity entity) {
        try {
            WcLabeldatasetEntity updatedEntity = service.updateEntity(id, entity);
            return new ResponseEntity<>(updatedEntity, HttpStatus.OK);
        } catch (IllegalArgumentException e) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteEntity(@PathVariable("id") int id) {
        try {
            service.deleteEntity(id);
            return new ResponseEntity<>(HttpStatus.NO_CONTENT);
        } catch (IllegalArgumentException e) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }
    @GetMapping("/getRecordsToLabel")
    public List<Long> getRecordsToLabel() {
        return service.getRecordsToLabel();
    }

    @GetMapping("/getRecordToLabel")
    public ResponseEntity<?> getRecordToLabel() {
        Optional<WcLabeldatasetEntity> recordToLabel = service.getEntityToLabel();
        if (recordToLabel.isEmpty()) {
            return ResponseEntity.notFound().build(); // Manejar el caso en que no hay registros no etiquetados
        } else {
            return ResponseEntity.ok(recordToLabel); // Devolver el registro aleatorio encontrado
        }
    }
}
