package tfg.windcaddy.Backend.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import tfg.windcaddy.Backend.domain.WcSpotEntity;
import tfg.windcaddy.Backend.services.WcSpotService;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/api/wcspots")
public class WcSpotController {
    private final WcSpotService wcSpotService;

    @Autowired
    public WcSpotController(WcSpotService wcSpotService) {
        this.wcSpotService = wcSpotService;
    }

    @GetMapping
    public ResponseEntity<List<WcSpotEntity>> getAllWcSpots() {
        List<WcSpotEntity> wcSpots = wcSpotService.getAllWcSpots();
        return ResponseEntity.ok(wcSpots);
    }

    @GetMapping("/{id}")
    public ResponseEntity<WcSpotEntity> getWcSpotById(@PathVariable int id) {
        Optional<WcSpotEntity> wcSpot = wcSpotService.getWcSpotById(id);
        return wcSpot.map(ResponseEntity::ok).orElseGet(() -> ResponseEntity.notFound().build());
    }

    @PostMapping
    public ResponseEntity<WcSpotEntity> createWcSpot(@RequestBody WcSpotEntity wcSpot) {
        WcSpotEntity createdWcSpot = wcSpotService.saveWcSpot(wcSpot);
        return ResponseEntity.status(HttpStatus.CREATED).body(createdWcSpot);
    }

    @PutMapping("/{id}")
    public ResponseEntity<WcSpotEntity> updateWcSpot(@PathVariable int id, @RequestBody WcSpotEntity updatedWcSpot) {
        if (wcSpotService.getWcSpotById(id).isEmpty()) {
            return ResponseEntity.notFound().build();
        }
        updatedWcSpot.setId(id);
        WcSpotEntity savedWcSpot = wcSpotService.saveWcSpot(updatedWcSpot);
        return ResponseEntity.ok(savedWcSpot);
    }

    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteWcSpot(@PathVariable int id) {
        if (wcSpotService.getWcSpotById(id).isEmpty()) {
            return ResponseEntity.notFound().build();
        }
        wcSpotService.deleteWcSpotById(id);
        return ResponseEntity.noContent().build();
    }
}
